{ open Parser }

let digit = ['0' - '9']
let digits = digit+
let ascii  = ([' '-'!' '#'-'[' ']'-'~'])
let esc    = '\\' ['\\' ''' '"' 'n' 'r' 't']

rule token =
  parse
  (*white space*)
  [' ' '\t' '\r' '\n'] { token lexbuf }

  (*Punctuators*)
  | '[' { OPENSQ }
  | ']' { CLOSESQ }
  | '(' { OPENPAR }
  | ')' { CLOSEPAR }
  | '{' { OPENBLOCK }
  | '}' { CLOSEBLOCK }
  | ';' { SEMICOLON }
  | ',' { COMMA }

  (*arithmetic*)
  | '+' { PLUS }
  | '-' { MINUS }
  | '*' { TIMES }
  | '/' { DIVIDE }
  | '%' { MODULO }

  (*unary operators *)
  | '!' { NOT }

  (*Binary Operators*)
  | '|' { BOR }
  | '&'	{ BAND }
  | ">>" { BRIGHT }
  | "<<" { BLEFT }
  | '^'	 { BXOR }
  | "!=" { NOTEQ }
  | "==" { EQ }

  (*key words*)
  | "resolve" { RESOLVE }
  | "reject" { REJECT }
  | "Action"	{ ACT }
  | "if" { IF }
  | "for" { FOR }
  | "true" { BOOL_LIT (true) }
  | "false" { BOOL_LIT (false) }
  | "else" { ELSE }
  | "while" { WHILE }
  | "break" { BREAK }
  | "continue" { CONTINUE }
  | "return" { RETURN }

  (* logical operators*)
  | "&&" { AND }
  | "||" { OR }
  | '!' { NOT }
  | '<' { LT }
  | "<="  { LTE }
  | '>'   { GT }
  | ">=" { GTE }


  (*special operators*)
  | "->" { BIND }
  | "[]" { ARRDEC }

  (*Assignment*)
  | "=" { ASSIGN }

	(*data types*)
	| "int" { INT }
  | "float" { FLOAT }
  | "bool" { BOOL }
  | "String" { STRING }
  | "void"    { VOID }
	| "Contract" { CON }
  | "Listener" { LIS }

  (*data values*)
  | "null" { NULL }
  | eof { EOF }

  (*literals*)
  | digits as lit { INT_LIT(int_of_string lit) }
  | digits '.'  digit* ( ['e' 'E'] ['+' '-']? digits )? as lit { FLOAT_LIT(float_of_string lit)}
  | '"' ((ascii | esc)* as s)'"' { STRING_LIT(s) }
  | ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as id { VARIABLE(id) }
	| eof { EOF }
